@@ -27,11 +27,6 @@ import org.apache.commons.net.ftp.FTPFile;
  * Implementation of FTPFileEntryParser and FTPFileListParser for IBM zOS/MVS
  * Systems.
  *
- * @author <a href="henrik.sorensen@balcab.ch">Henrik Sorensen</a>
- * @author <a href="jnadler@srcginc.com">Jeff Nadler</a>
- * @author <a href="wnoto@openfinance.com">William Noto</a>
- *
- * @version $Id: MVSFTPEntryParser.java 1081476 2011-03-14 17:09:59Z sebb $
  * @see org.apache.commons.net.ftp.FTPFileEntryParser FTPFileEntryParser (for
  *      usage instructions)
  */
@@ -59,17 +54,25 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
                                                                     // 13:52
 
     /**
-     * Matches these entries: Volume Unit Referred Ext Used Recfm Lrecl BlkSz
-     * Dsorg Dsname B10142 3390 2006/03/20 2 31 F 80 80 PS MDI.OKL.WORK
+     * Matches these entries:
      *
+     * <pre>
+     *  Volume Unit    Referred Ext Used Recfm Lrecl BlkSz Dsorg Dsname
+     *  B10142 3390   2006/03/20  2   31  F       80    80  PS   MDI.OKL.WORK
+     * </pre>
+     *
+     * @see <a href=
+     *      "https://www.ibm.com/support/knowledgecenter/zosbasics/com.ibm.zos.zconcepts/zconcepts_159.htm">Data
+     *      set record formats</a>
      */
     static final String FILE_LIST_REGEX = "\\S+\\s+" + // volume
                                                                 // ignored
             "\\S+\\s+" + // unit - ignored
             "\\S+\\s+" + // access date - ignored
             "\\S+\\s+" + // extents -ignored
-            "\\S+\\s+" + // used - ignored
-            "[FV]\\S*\\s+" + // recfm - must start with F or V
+            // If the values are too large, the fields may be merged (NET-639)
+            "(?:\\S+\\s+)?" + // used - ignored
+            "(?:F|FB|V|VB|U)\\s+" + // recfm - F[B], V[B], U
             "\\S+\\s+" + // logical record length -ignored
             "\\S+\\s+" + // block size - ignored
             "(PS|PO|PO-E)\\s+" + // Dataset organisation. Many exist
@@ -77,8 +80,11 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
             "(\\S+)\\s*"; // Dataset Name (file name)
 
     /**
-     * Matches these entries: Name VV.MM Created Changed Size Init Mod Id
-     * TBSHELF 01.03 2002/09/12 2002/10/11 09:37 11 11 0 KIL001
+     * Matches these entries:
+     * <pre>
+     *   Name      VV.MM   Created       Changed      Size  Init   Mod   Id
+     *   TBSHELF   01.03 2002/09/12 2002/10/11 09:37    11    11     0 KIL001
+     * </pre>
      */
     static final String MEMBER_LIST_REGEX = "(\\S+)\\s+" + // name
             "\\S+\\s+" + // version, modification (ignored)
@@ -91,12 +97,15 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
             "\\S+\\s*"; // id of user who modified (ignored)
 
     /**
-     * Matches these entries, note: no header: IBMUSER1 JOB01906 OUTPUT 3 Spool
-     * Files 012345678901234567890123456789012345678901234 1 2 3 4
+     * Matches these entries, note: no header:
+     * <pre>
+     *   IBMUSER1  JOB01906  OUTPUT    3 Spool Files
+     *   012345678901234567890123456789012345678901234
+     *             1         2         3         4
+     * </pre>
      */
-    static final String JES_LEVEL_1_LIST_REGEX = "(\\S+)\\s+" + // job
-                                                                        // name
-                                                                        // ignored
+    static final String JES_LEVEL_1_LIST_REGEX =
+            "(\\S+)\\s+" + // job name ignored
             "(\\S+)\\s+" + // job number
             "(\\S+)\\s+" + // job status (OUTPUT,INPUT,ACTIVE)
             "(\\S+)\\s+" + // number of spool files
@@ -105,16 +114,41 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
     ;
 
     /**
-     * JES INTERFACE LEVEL 2 parser Matches these entries: JOBNAME JOBID OWNER
-     * STATUS CLASS IBMUSER1 JOB01906 IBMUSER OUTPUT A RC=0000 3 spool files
-     * IBMUSER TSU01830 IBMUSER OUTPUT TSU ABEND=522 3 spool files
-     * 012345678901234567890123456789012345678901234 1 2 3 4
-     * 012345678901234567890123456789012345678901234567890
+     * JES INTERFACE LEVEL 2 parser
+     * Matches these entries:
+     * <pre>
+     * JOBNAME  JOBID    OWNER    STATUS CLASS
+     * IBMUSER1 JOB01906 IBMUSER  OUTPUT A        RC=0000 3 spool files
+     * IBMUSER  TSU01830 IBMUSER  OUTPUT TSU      ABEND=522 3 spool files
+     * </pre>
+     * Sample output from FTP session:
+     * <pre>
+     * ftp> quote site filetype=jes
+     * 200 SITE command was accepted
+     * ftp> ls
+     * 200 Port request OK.
+     * 125 List started OK for JESJOBNAME=IBMUSER*, JESSTATUS=ALL and JESOWNER=IBMUSER
+     * JOBNAME  JOBID    OWNER    STATUS CLASS
+     * IBMUSER1 JOB01906 IBMUSER  OUTPUT A        RC=0000 3 spool files
+     * IBMUSER  TSU01830 IBMUSER  OUTPUT TSU      ABEND=522 3 spool files
+     * 250 List completed successfully.
+     * ftp> ls job01906
+     * 200 Port request OK.
+     * 125 List started OK for JESJOBNAME=IBMUSER*, JESSTATUS=ALL and JESOWNER=IBMUSER
+     * JOBNAME  JOBID    OWNER    STATUS CLASS
+     * IBMUSER1 JOB01906 IBMUSER  OUTPUT A        RC=0000
+     * --------
+     * ID  STEPNAME PROCSTEP C DDNAME   BYTE-COUNT
+     * 001 JES2              A JESMSGLG       858
+     * 002 JES2              A JESJCL         128
+     * 003 JES2              A JESYSMSG       443
+     * 3 spool files
+     * 250 List completed successfully.
+     * </pre>
      */
 
-    static final String JES_LEVEL_2_LIST_REGEX = "(\\S+)\\s+" + // job
-                                                                        // name
-                                                                        // ignored
+    static final String JES_LEVEL_2_LIST_REGEX =
+            "(\\S+)\\s+" + // job name ignored
             "(\\S+)\\s+" + // job number
             "(\\S+)\\s+" + // owner ignored
             "(\\S+)\\s+" + // job status (OUTPUT,INPUT,ACTIVE) ignored
@@ -124,12 +158,12 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
 
     /*
      * ---------------------------------------------------------------------
-     * Very brief and incomplete description of the zOS/MVS-filesystem. (Note:
+     * Very brief and incomplete description of the zOS/MVS-file system. (Note:
      * "zOS" is the operating system on the mainframe, and is the new name for
      * MVS)
      *
-     * The filesystem on the mainframe does not have hierarchal structure as for
-     * example the unix filesystem. For a more comprehensive description, please
+     * The file system on the mainframe does not have hierarchal structure as for
+     * example the unix file system. For a more comprehensive description, please
      * refer to the IBM manuals
      *
      * @LINK:
@@ -169,8 +203,8 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
      * for the FTP transfer.
      *
      * Any one beginning with either F or V can safely used by FTP transfer. All
-     * others should only be used with great care, so this version will just
-     * ignore the other record formats. F means a fixed number of records per
+     * others should only be used with great care.
+     * F means a fixed number of records per
      * allocated storage, and V means a variable number of records.
      *
      *
@@ -184,7 +218,7 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
      * Implementation notes ====================
      *
      * Only datasets that have dsorg PS, PO or PO-E and have recfm beginning
-     * with F or V, is fully parsed.
+     * with F or V or U, is fully parsed.
      *
      * The following fields in FTPFile is used: FTPFile.Rawlisting: Always set.
      * FTPFile.Type: DIRECTORY_TYPE or FILE_TYPE or UNKNOWN FTPFile.Name: name
@@ -229,33 +263,26 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
      *            A line of text from the file listing
      * @return An FTPFile instance corresponding to the supplied entry
      */
-    public FTPFile parseFTPEntry(String entry) {
-        boolean isParsed = false;
-        FTPFile f = new FTPFile();
-
-        if (isType == FILE_LIST_TYPE)
-            isParsed = parseFileList(f, entry);
-        else if (isType == MEMBER_LIST_TYPE) {
-            isParsed = parseMemberList(f, entry);
-            if (!isParsed)
-                isParsed = parseSimpleEntry(f, entry);
+    @Override
+    public FTPFile parseFTPEntry(final String entry) {
+        if (isType == FILE_LIST_TYPE) {
+            return parseFileList(entry);
+        } else if (isType == MEMBER_LIST_TYPE) {
+            return parseMemberList(entry);
         } else if (isType == UNIX_LIST_TYPE) {
-            isParsed = parseUnixList(f, entry);
+             return unixFTPEntryParser.parseFTPEntry(entry);
         } else if (isType == JES_LEVEL_1_LIST_TYPE) {
-            isParsed = parseJeslevel1List(f, entry);
+            return parseJeslevel1List(entry);
         } else if (isType == JES_LEVEL_2_LIST_TYPE) {
-            isParsed = parseJeslevel2List(f, entry);
+            return parseJeslevel2List(entry);
         }
 
-        if (!isParsed)
-            f = null;
-
-        return f;
+        return null;
     }
 
     /**
      * Parse entries representing a dataset list. Only datasets with DSORG PS or
-     * PO or PO-E and with RECFM F* or V* will be parsed.
+     * PO or PO-E and with RECFM F[B], V[B], U will be parsed.
      *
      * Format of ZOS/MVS file list: 1 2 3 4 5 6 7 8 9 10 Volume Unit Referred
      * Ext Used Recfm Lrecl BlkSz Dsorg Dsname B10142 3390 2006/03/20 2 31 F 80
@@ -273,16 +300,15 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
      * a tape archive. These entries is currently not supported by this parser.
      * A null value is returned.
      *
-     * @param file
-     *            will be updated with Name, Type, Timestamp if parsed.
      * @param entry zosDirectoryEntry
-     * @return true: entry was parsed, false: entry was not parsed.
+     * @return null: entry was not parsed.
      */
-    private boolean parseFileList(FTPFile file, String entry) {
+    private FTPFile parseFileList(final String entry) {
         if (matches(entry)) {
+            final FTPFile file = new FTPFile();
             file.setRawListing(entry);
-            String name = group(2);
-            String dsorg = group(1);
+            final String name = group(2);
+            final String dsorg = group(1);
             file.setName(name);
 
             // DSORG
@@ -294,159 +320,159 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
                 file.setType(FTPFile.DIRECTORY_TYPE);
             }
             else {
-                return false;
+                return null;
             }
 
-            return true;
+            return file;
         }
 
-        return false;
+        return null;
     }
 
     /**
      * Parse entries within a partitioned dataset.
      *
-     * Format of a memberlist within a PDS: 1 2 3 4 5 6 7 8 9 Name VV.MM Created
-     * Changed Size Init Mod Id TBSHELF 01.03 2002/09/12 2002/10/11 09:37 11 11
-     * 0 KIL001 TBTOOL 01.12 2002/09/12 2004/11/26 19:54 51 28 0 KIL001
+     * Format of a memberlist within a PDS:
+     * <pre>
+     *    0         1        2          3        4     5     6      7    8
+     *   Name      VV.MM   Created       Changed      Size  Init   Mod   Id
+     *   TBSHELF   01.03 2002/09/12 2002/10/11 09:37    11    11     0 KIL001
+     *   TBTOOL    01.12 2002/09/12 2004/11/26 19:54    51    28     0 KIL001
      *
-     * ------------------------------------------- [1] Name [2] VV.MM: Version .
-     * modification [3] Created: yyyy / MM / dd [4,5] Changed: yyyy / MM / dd
-     * HH:mm [6] Size: number of lines [7] Init: number of lines when first
-     * created [8] Mod: number of modified lines a last save [9] Id: User id for
-     * last update
+     * -------------------------------------------
+     * [1] Name
+     * [2] VV.MM: Version . modification
+     * [3] Created: yyyy / MM / dd
+     * [4,5] Changed: yyyy / MM / dd HH:mm
+     * [6] Size: number of lines
+     * [7] Init: number of lines when first created
+     * [8] Mod: number of modified lines a last save
+     * [9] Id: User id for last update
+     * </pre>
      *
-     *
-     * @param file
-     *            will be updated with Name, Type and Timestamp if parsed.
      * @param entry zosDirectoryEntry
-     * @return true: entry was parsed, false: entry was not parsed.
+     * @return null: entry was not parsed.
      */
-    private boolean parseMemberList(FTPFile file, String entry) {
+    private FTPFile parseMemberList(final String entry) {
+        final FTPFile file = new FTPFile();
         if (matches(entry)) {
             file.setRawListing(entry);
-            String name = group(1);
-            String datestr = group(2) + " " + group(3);
+            final String name = group(1);
+            final String datestr = group(2) + " " + group(3);
             file.setName(name);
             file.setType(FTPFile.FILE_TYPE);
             try {
                 file.setTimestamp(super.parseTimestamp(datestr));
-            } catch (ParseException e) {
-                e.printStackTrace();
+            } catch (final ParseException e) {
                 // just ignore parsing errors.
                 // TODO check this is ok
-                return false; // this is a parsing failure too.
+                // Drop thru to try simple parser
             }
-            return true;
+            return file;
         }
 
-        return false;
-    }
-
-    /**
-     * Assigns the name to the first word of the entry. Only to be used from a
-     * safe context, for example from a memberlist, where the regex for some
-     * reason fails. Then just assign the name field of FTPFile.
-     *
-     * @param file
-     * @param entry
-     * @return
-     */
-    private boolean parseSimpleEntry(FTPFile file, String entry) {
-        if (entry != null && entry.length() > 0) {
+        /*
+         * Assigns the name to the first word of the entry. Only to be used from a
+         * safe context, for example from a memberlist, where the regex for some
+         * reason fails. Then just assign the name field of FTPFile.
+         */
+        if (entry != null && entry.trim().length() > 0) {
             file.setRawListing(entry);
-            String name = entry.split(" ")[0];
+            final String name = entry.split(" ")[0];
             file.setName(name);
             file.setType(FTPFile.FILE_TYPE);
-            return true;
+            return file;
         }
-        return false;
+        return null;
     }
 
     /**
-     * Parse the entry as a standard unix file. Using the UnixFTPEntryParser.
+     * Matches these entries, note: no header:
+     * <pre>
+     * [1]      [2]      [3]   [4] [5]
+     * IBMUSER1 JOB01906 OUTPUT 3 Spool Files
+     * 012345678901234567890123456789012345678901234
+     *           1         2         3         4
+     * -------------------------------------------
+     * Group in regex
+     * [1] Job name
+     * [2] Job number
+     * [3] Job status (INPUT,ACTIVE,OUTPUT)
+     * [4] Number of sysout files
+     * [5] The string "Spool Files"
+     *</pre>
      *
-     * @param file
-     * @param entry
-     * @return true: entry is parsed, false: entry could not be parsed.
-     */
-    private boolean parseUnixList(FTPFile file, String entry) {
-        file = unixFTPEntryParser.parseFTPEntry(entry);
-        if (file == null)
-            return false;
-        return true;
-    }
-
-    /**
-     * Matches these entries, note: no header: [1] [2] [3] [4] [5] IBMUSER1
-     * JOB01906 OUTPUT 3 Spool Files
-     * 012345678901234567890123456789012345678901234 1 2 3 4
-     * ------------------------------------------- Group in regex [1] Job name
-     * [2] Job number [3] Job status (INPUT,ACTIVE,OUTPUT) [4] Number of sysout
-     * files [5] The string "Spool Files"
-     *
-     *
-     * @param file
-     *            will be updated with Name, Type and Timestamp if parsed.
      * @param entry zosDirectoryEntry
-     * @return true: entry was parsed, false: entry was not parsed.
+     * @return null: entry was not parsed.
      */
-    private boolean parseJeslevel1List(FTPFile file, String entry) {
+    private FTPFile parseJeslevel1List(final String entry) {
         if (matches(entry)) {
+            final FTPFile file = new FTPFile();
             if (group(3).equalsIgnoreCase("OUTPUT")) {
                 file.setRawListing(entry);
-                String name = group(2); /* Job Number, used by GET */
+                final String name = group(2); /* Job Number, used by GET */
                 file.setName(name);
                 file.setType(FTPFile.FILE_TYPE);
-                return true;
+                return file;
             }
         }
 
-        return false;
+        return null;
     }
 
     /**
-     * Matches these entries, note: no header: [1] [2] [3] [4] [5] JOBNAME JOBID
-     * OWNER STATUS CLASS IBMUSER1 JOB01906 IBMUSER OUTPUT A RC=0000 3 spool
-     * files IBMUSER TSU01830 IBMUSER OUTPUT TSU ABEND=522 3 spool files
-     * 012345678901234567890123456789012345678901234 1 2 3 4
-     * ------------------------------------------- Group in regex [1] Job name
-     * [2] Job number [3] Owner [4] Job status (INPUT,ACTIVE,OUTPUT) [5] Job
-     * Class [6] The rest
+     * Matches these entries:
+     * <pre>
+     * [1]      [2]      [3]     [4]    [5]
+     * JOBNAME  JOBID    OWNER   STATUS CLASS
+     * IBMUSER1 JOB01906 IBMUSER OUTPUT A       RC=0000 3 spool files
+     * IBMUSER  TSU01830 IBMUSER OUTPUT TSU     ABEND=522 3 spool files
+     * 012345678901234567890123456789012345678901234
+     *           1         2         3         4
+     * -------------------------------------------
+     * Group in regex
+     * [1] Job name
+     * [2] Job number
+     * [3] Owner
+     * [4] Job status (INPUT,ACTIVE,OUTPUT)
+     * [5] Job Class
+     * [6] The rest
+     * </pre>
      *
-     *
-     * @param file
-     *            will be updated with Name, Type and Timestamp if parsed.
      * @param entry zosDirectoryEntry
-     * @return true: entry was parsed, false: entry was not parsed.
+     * @return null: entry was not parsed.
      */
-    private boolean parseJeslevel2List(FTPFile file, String entry) {
+    private FTPFile parseJeslevel2List(final String entry) {
         if (matches(entry)) {
+            final FTPFile file = new FTPFile();
             if (group(4).equalsIgnoreCase("OUTPUT")) {
                 file.setRawListing(entry);
-                String name = group(2); /* Job Number, used by GET */
+                final String name = group(2); /* Job Number, used by GET */
                 file.setName(name);
                 file.setType(FTPFile.FILE_TYPE);
-                return true;
+                return file;
             }
         }
 
-        return false;
+        return null;
     }
 
     /**
      * preParse is called as part of the interface. Per definition is is called
-     * before the parsing takes place. Three kind of lists is recognize:
-     * z/OS-MVS File lists z/OS-MVS Member lists unix file lists
+     * before the parsing takes place.
+     * Three kind of lists is recognize:
+     * z/OS-MVS File lists
+     * z/OS-MVS Member lists
+     * unix file lists
      * @since 2.0
      */
     @Override
-    public List<String> preParse(List<String> orig) {
+    public List<String> preParse(final List<String> orig) {
         // simply remove the header line. Composite logic will take care of the
         // two different types of
         // list in short order.
         if (orig != null && orig.size() > 0) {
-            String header = orig.get(0);
+            final String header = orig.get(0);
             if (header.indexOf("Volume") >= 0 && header.indexOf("Dsname") >= 0) {
                 setType(FILE_LIST_TYPE);
                 super.setRegex(FILE_LIST_REGEX);
@@ -479,7 +505,7 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
      * Explicitly set the type of listing being processed.
      * @param type The listing type.
      */
-    void setType(int type) {
+    void setType(final int type) {
         isType = type;
     }
 
@@ -489,7 +515,7 @@ public class MVSFTPEntryParser extends ConfigurableFTPFileEntryParserImpl {
     @Override
     protected FTPClientConfig getDefaultConfiguration() {
         return new FTPClientConfig(FTPClientConfig.SYST_MVS,
-                DEFAULT_DATE_FORMAT, null, null, null, null);
+                DEFAULT_DATE_FORMAT, null);
     }
 
 }
